package main

import (
	"database/sql"
	"fmt"

	migration "bitbucket.org/prmunty/util/dbmigration/db"
	sqlite "bitbucket.org/prmunty/util/dbmigration/sqliteutils"

	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 2.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {

		return DoUntilErr(tx,
			func(tx *sql.Tx) error {
				return RealTimeTrendingTable(tx, "condenser")
			},
			func(tx *sql.Tx) error {
				return RealTimeTrendingTable(tx, "vessel")
			},
			EquipmentTableUpdate,
			TestPanelIPs,
			VesselFreezeDataUpdate,
			SetSoftwareVersion,
		)

	})
}

func DoUntilErr(tx *sql.Tx, funcs ...func(tx *sql.Tx) error) (err error) {
	for _, fn := range funcs {
		if err = fn(tx); err != nil {
			break
		}
	}
	return err
}
func RealTimeTrendingTable(tx *sql.Tx, product string) (err error) {
	if !(sqlite.DB{tx}).TableExists("trending_real_time_" + product) {
		_, err = tx.Exec(fmt.Sprintf(`
            CREATE TABLE trending_real_time_%s(
                   language_id Integer,
                   analog_data Integer,
                   unit Text,
                   text_index Integer);
            `, product))
	}

	return err
}

func Apply(tx *sql.Tx, statements ...string) (e error) {
	for _, statement := range statements {
		if _, e = tx.Exec(statement); e != nil {
			return e
		}
	}
	return e
}

func EquipmentTableUpdate(tx *sql.Tx) (err error) {
	var (
		col sqlite.Column
		db  = sqlite.DB{tx}
	)

	if col, err = db.GetColumn("equipment", "product_name"); err != nil {
		return err
	}

	if col.PrimaryKey {
		err = Apply(tx,
			`CREATE TABLE tmp_equipment(
                    product_name TEXT,
                    ip TEXT UNIQUE,
                    port VARCHAR(10),
                    type_name VARCHAR(25),
                    external_address TEXT);`,

			`INSERT INTO tmp_equipment
                    SELECT product_name, ip, port, product_name as type_name, external_address
                    FROM equipment;`,

			`CREATE TABLE tmp_equipment_components(
                    host_id INTEGER,
                    component_name TEXT,
                    type_name VARCHAR(25),
                    CONSTRAINT component UNIQUE(host_id, type_name),
                    FOREIGN KEY(host_id) REFERENCES equipment(rowid));`,

			`INSERT INTO tmp_equipment_components
                    SELECT tmp_equipment.rowid as host_id,
                           equipment_components.component_id as component_name,
                           equipment_components.component_id as type_name
                    FROM tmp_equipment JOIN equipment_components USING(product_name);`,

			"DROP TABLE equipment;",

			"DROP TABLE equipment_components;",

			"ALTER TABLE tmp_equipment RENAME TO equipment;",

			"ALTER TABLE tmp_equipment_components RENAME TO equipment_components;",
		)
	}
	return err
}

func TestPanelIPs(tx *sql.Tx) (err error) {
	return Apply(tx,
		`DROP TABLE IF EXISTS test_panel_ips;`,

		`CREATE TABLE test_panel_ips (
                type_name VARCHAR(25),
                test_letter CHARACTER(1),
                ip TEXT
                );`,

		`INSERT INTO test_panel_ips VALUES('compressor',       'A', '10.14.228.114');
        INSERT INTO test_panel_ips VALUES('condenser/vessel', 'A', '10.14.228.116');
        INSERT INTO test_panel_ips VALUES('evaporator',       'A', '10.14.228.117');
        INSERT INTO test_panel_ips VALUES('engine_room',      'A', '10.14.228.118');
        INSERT INTO test_panel_ips VALUES('compressor',       'B', '10.14.228.119');
        INSERT INTO test_panel_ips VALUES('condenser/vessel', 'B', '10.14.228.121');
        INSERT INTO test_panel_ips VALUES('evaporator',       'B', '10.14.228.122');
        INSERT INTO test_panel_ips VALUES('engine_room',      'B', '10.14.228.128');`,
	)
}

func VesselFreezeDataUpdate(tx *sql.Tx) (err error) {
	var (
		db = sqlite.DB{tx}
	)
	if db.ColumnExists("freeze_data_vessel", "section") {
		return err
	}
	return Apply(tx,
		`DROP TABLE IF EXISTS freeze_data_vessel_new;

         CREATE TABLE freeze_data_vessel_new(
                freeze_id INTEGER,    
                section INTEGER, 
                selected BOOLEAN,    
                PRIMARY KEY (freeze_id, section));

        INSERT INTO freeze_data_vessel_new VALUES(2085, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2090, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2091, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2092, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2093, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2094, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2095, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2096, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2097, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2249, 1, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2257, 1, 0);
        
        INSERT INTO freeze_data_vessel_new VALUES(2085, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2090, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2091, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2092, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2093, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2094, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2095, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2096, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2097, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2249, 2, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2257, 2, 0);
        
        INSERT INTO freeze_data_vessel_new VALUES(2085, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2090, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2091, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2092, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2093, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2094, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2095, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2096, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2097, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2249, 3, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2257, 3, 0);

        INSERT INTO freeze_data_vessel_new VALUES(134, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(135, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(136, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(137, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(138, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(139, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(140, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(141, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(142, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(143, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2343, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2344, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2345, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2534, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2535, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2536, 4, 0);
        INSERT INTO freeze_data_vessel_new VALUES(2537, 4, 0);
     
     
     
        UPDATE freeze_data_vessel_new set selected = (SELECT freeze_data_vessel.selected
                                                    FROM freeze_data_vessel
                                                    WHERE freeze_data_vessel.freeze_id = freeze_data_vessel.freeze_id);
     
       DROP TABLE freeze_data_vessel;
     
       ALTER TABLE freeze_data_vessel_new RENAME TO freeze_data_vessel;

`)
}

func SetSoftwareVersion(tx *sql.Tx) (err error) {
	return Apply(tx, `UPDATE software SET version = '10.30', release = '4/15/2016';`)
}
