// Coincides with 10.41
package main

import (
	"database/sql"
	"fmt"
	"strings"

	"bitbucket.org/prmunty/middleware/auth"
	migration "bitbucket.org/prmunty/util/dbmigration/db"
	sqlite "bitbucket.org/prmunty/util/dbmigration/sqliteutils"
	_ "github.com/mattn/go-sqlite3"
)

func main() {
	const version float64 = 5.0
	const TempUsersTable = "users_migration_table"

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		col, err := sqlite.DB{tx}.GetColumn("user", "id")
		if err != nil || (strings.ToUpper(string(col.DataType)) != "TEXT") {
			err = migration.DoUntilErr(tx,
				BuildTempUsersTable(TempUsersTable),
				UpdateUsersSchema(TempUsersTable),
				UpdateUserActionsSchema(TempUsersTable, "condenser"),
				UpdateUserActionsSchema(TempUsersTable, "vessel"),
				UpdateUserDefinedSchema(TempUsersTable, "condenser"),
				UpdateUserDefinedSchema(TempUsersTable, "vessel"),
				RemoveMigrationTable(TempUsersTable),
			)
		}

		return err
	})

}

type fieldName string
type fieldNames []fieldName

func (fs fieldNames) String() string {
	s := ""
	for _, f := range fs {
		s += string(f) + ", "
	}

	return strings.TrimSuffix(s, ", ")
}

type User struct {
	OldID int
	auth.User
}

// type field struct {
// 	name       fieldName
// 	fieldType  string
// 	constraint string
// }

// type fields []fields

// func (fs fields) String() string {
// 	s := ""
// 	for _, f := range fs {
// 		s = s + string(f.name) + " " + f.fieldType + " " + f.constraint + ", "
// 	}
// 	return strings.TrimSuffix(s, ", ")
// }

// func (fs fields) Names() fieldNames {
// 	names := make(fieldNames, 0, len(fs))
// 	for _, f := range fields {
// 		names = append(names, f.name)
// 	}

// }

func BuildTempUsersTable(tableName string) func(tx *sql.Tx) (err error) {
	return func(tx *sql.Tx) (err error) {
		users, err := getUsers(tx)
		if err != nil {
			return err
		}
		stmt := fmt.Sprintf(`DROP TABLE IF EXISTS %[1]s;
                         CREATE TABLE %[1]s(
                             old_user_id INTEGER,
                             new_user_id TEXT);`,
			tableName)

		_, err = tx.Exec(stmt)
		if err != nil {
			return err
		}

		for _, u := range users {
			_, err := tx.Exec(fmt.Sprintf(`INSERT INTO %s (old_user_id, new_user_id) VALUES(?,?);`, tableName), u.OldID, u.User.Id())
			if err != nil {
				return err
			}
		}

		return err

	}
}

func getUsers(tx *sql.Tx) (users []User, err error) {
	rows, err := tx.Query(`SELECT id, firstName, lastName, email, phone, carrier, pin, secLevel FROM users`)
	if err != nil {
		return users, err
	}
	defer rows.Close()
	for rows.Next() {
		user := User{}
		err := rows.Scan(&user.OldID, &user.FirstName, &user.LastName, &user.Email, &user.Phone, &user.Carrier, &user.Pin, &user.SecLevel)
		if err != nil {
			return users, err
		}
		users = append(users, user)
	}

	return users, rows.Err()
}

func UpdateUsersSchema(migrationTable string) func(*sql.Tx) error {
	return updateTable(migrationTable, "users", temporaryUsersTable)

}

func temporaryUsersTable(tx *sql.Tx, migrationTable, tableName string) (err error) {
	_, err = tx.Exec(fmt.Sprintf(`CREATE TABLE %s(
       -- ID IS GENERATED BUILDING A HASH FROM THE NAME EMAIL PHONE AND SECLEVEL
                 id TEXT PRIMARY KEY,
                 firstName TEXT,
                 lastName TEXT,
                 email TEXT,
                 phone TEXT,
                 carrier TEXT,
                 pin TEXT UNIQUE,
                 secLevel INTEGER DEFAULT 0);`, tableName))
	if err != nil {
		return err
	}
	return migrateUserID(tx, "users", tableName, migrationTable, "id", "firstName", "lastName", "email", "phone", "carrier", "pin", "secLevel")

}

func UpdateUserActionsSchema(tempUsersTable, product string) func(tx *sql.Tx) (err error) {
	return updateTable(
		tempUsersTable,
		"user_actions_"+product,
		temporaryUserActionsTable(product))
}

func temporaryUserActionsTable(product string) func(*sql.Tx, string, string) (err error) {
	var tableName = "user_actions"
	if product != "" {
		tableName = tableName + "_" + product
	}

	return func(tx *sql.Tx, tempUsersTable, tempName string) error {
		_, err := tx.Exec(fmt.Sprintf(`CREATE TABLE %s(
       user_id TEXT,
       action_data TEXT,
       time_stamp TIMESTAMP,
       FOREIGN KEY(user_id) REFERENCES users(id));`, tempName))
		if err != nil {
			return err
		}

		return migrateUserID(tx, tableName, tempName, tempUsersTable, "user_id", "action_data", "time_stamp")
	}

}

func UpdateMaintenanceLogsSchema(migrationTable string) func(tx *sql.Tx) (err error) {
	return updateTable(migrationTable, "global_settings_maintenance_logs", temporaryMaintenanceLogsTable)
}

func temporaryMaintenanceLogsTable(tx *sql.Tx, migrationTable, tableName string) (err error) {
	_, err = tx.Exec(fmt.Sprintf(`CREATE TABLE %s(
       user_id TEXT,
       log_date DATETIME,
       log_text TEXT,
       FOREIGN KEY(user_id) REFERENCES users(id));`, tableName))
	if err != nil {
		return err
	}
	return migrateUserID(tx, "global_settings_maintenance_logs", tableName, migrationTable, "user_id", "log_date", "log_text")

}

func UpdateUserDefinedSchema(migrationTable, product string) func(tx *sql.Tx) (err error) {
	return updateTable(
		migrationTable,
		"user_defined_"+product,
		temporaryUserDefinedSchema(product))
}

func temporaryUserDefinedSchema(product string) func(*sql.Tx, string, string) (err error) {
	tableName := "user_defined"
	if len(product) > 0 {
		tableName = tableName + "_" + product
	}
	return func(tx *sql.Tx, tempUsersTable, tempName string) (err error) {
		_, err = tx.Exec(fmt.Sprintf(`CREATE TABLE %s(
	id      Integer,
	user_id TEXT,
	page    Text,
    PRIMARY KEY ( id, user_id, page ));`, tempName))
		if err != nil {
			return err
		}

		return migrateUserID(tx, tableName, tempName, tempUsersTable, "user_id", "id", "page")
	}
}

func migrateUserID(tx *sql.Tx, oldTable, tempTable, migrationTable, idField string, fs ...fieldName) error {
	stmt := fmt.Sprintf(`INSERT INTO %[1]s
                                               SELECT %[2]s.new_user_id as %[3]s, %[4]v FROM %[5]s join %[2]s ON (%[5]s.%[3]s == %[2]s.old_user_id)`,
		tempTable,
		migrationTable,
		idField,
		fieldNames(fs),
		oldTable,
	)

	_, err := tx.Exec(stmt)
	return err

}

func updateTable(migrationTable, tableName string, tempTableBuild func(*sql.Tx, string, string) error) func(tx *sql.Tx) error {
	return func(tx *sql.Tx) (err error) {
		tempTableName := tableName + "_temp"
		if err = tempTableBuild(tx, migrationTable, tempTableName); err != nil {
			return err
		}
		return sqlite.DB{tx}.MoveTable(tempTableName, tableName)
	}
}

func RemoveMigrationTable(migrationTable string) func(*sql.Tx) error {
	return func(tx *sql.Tx) error {
		return sqlite.DB{tx}.DropTable(migrationTable)
	}
}
